require 'thor'
require 'date'

class Finatra < Thor
  include Thor::Actions

  def self.source_root
    File.expand_path(File.dirname(__FILE__))
  end

  desc "new <MyAppName>", "Create a new Finatra app"
  def new(name)
    @app_name = name.to_s.gsub(/[^a-zA-Z0-9]/,'')
    @dir_name = name.downcase.to_s.gsub(/[^a-zA-Z0-9]/,'')
    @org_name = ask 'Org Name (com.<username>)' while @org_name.to_s.empty?

    log "Creating #{@app_name} in #{Dir.pwd}..."

    copy_project_files
    create_controller
    create_controller_test
    create_template
  end

  desc "console", "Starts a scala console"
  def console
    exec "mvn scala:console"
  end

  desc "compile", "Compile your app with Maven"
  def compile
    exec "mvn compile"
  end

  desc "start", "Run your app on port 7070"
  def start
    exec "mvn scala:run"
  end

  desc "test", "Run the tests"
  def test
    exec "mvn test"
  end

  desc "package", "Package a jar for your app"
  def package
    exec "mvn package"
  end

  private

  def copy_project_files
    directory "templates/project", @dir_name
  end

  def create_controller
    template "templates/App.scala.tt", "#{src_dir}/#{@app_name}.scala"
  end

  def create_controller_test
    template "templates/Spec.scala.tt", "#{test_dir}/#{@app_name}Spec.scala"
  end

  def create_template
    template "templates/timeline.mustache.tt", "#{resource_dir}/timeline.mustache"
  end

  def src_dir
    "#{@dir_name}/src/main/scala/#{@org_name.gsub('.','/')}/#{@dir_name}"
  end

  def resource_dir
    "#{@dir_name}/src/main/resources"
  end

  def test_dir
    "#{@dir_name}/src/test/scala/#{@org_name.gsub('.','/')}/#{@dir_name}"
  end

  def log(msg)
    puts "[finatra] :: #{msg}"
  end
end
